// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "driver.h"

static void *startRecordThread(void *pv);

bool VPBTrunk::ringHandler(TrunkEvent *event)
{
	char script[256];
	int argc = 0;
	char *args[2];
	char **start = NULL;
	const char *data;
	const char *value;
	TrunkGroup *grp = NULL;

	switch(event->id)
	{
	case TRUNK_ENTER_STATE:
		debug->DebugState(this, "ringing");
		status[id] = 'r';
		flags.dsp = DSP_MODE_CALLERID;
		return true;
	case TRUNK_START_SCRIPT:
		return false;
	case TRUNK_RING_START:
		flags.dsp = DSP_MODE_INACTIVE;
		return idleHandler(event);
	case TRUNK_RINGING_ON:
		++rings;
		endTimer();
		return true;
	case TRUNK_RINGING_OFF:
		if(rings > 1)
			flags.dsp = DSP_MODE_INACTIVE;

		// DR 24/7/01 - start sampling CID
		if (rings == 1) {
			slog(SLOG_INFO)<<"start sampling CID" << endl;
			// start sampling CID
			pthread_create(&RecordCIDThread, NULL, 
				       startRecordThread,(void*)this);
		}

		// DR 24/7/01 - decode CID
		if (rings == 2) {
			char number[VPB_MAX_STR];
			int ret;

			// make sure recording has finished.....
			vpb_record_terminate(handle);

			strcpy(number, "pstn:"); 
			ret = vpb_cid_decode(number+5, cidbuf, CID_BUF_SZ);
			if (ret == VPB_OK) {
				setConstant(SYM_CLID, number + 5);
				setConstant(SYM_CALLER, number);
				slog(SLOG_INFO)<<"number = "<<number<<endl;
			}
			else {
				slog(SLOG_INFO) << "CID decode failed" <<
					"ret = " << ret <<endl;
			}
		}

		if(rings < group->getAnswer())
		{
			setTimer(group->getRingTime() * 1000);
			return true;
		}
		sprintf(script, "ring%d", event->parm.ring.digit);
		if(!grp)
			grp = group;
		start = getInitial(args);
		if(Attach(*start))
		{
			setList(++start);
			handler = &VPBTrunk::stepHandler;
			return true;
		}
		rings = 0;
		slog(SLOG_ERROR) << "vpb" << id << ": cannot answer call" << endl;	
		return true;
	case TRUNK_LINE_WINK:
		if(!flags.offhook)
			return true;
	case TRUNK_STOP_DISCONNECT:
	case TRUNK_CPA_DIALTONE:
	case TRUNK_TIMER_EXPIRED:
		flags.dsp = DSP_MODE_INACTIVE;
		handler = &VPBTrunk::idleHandler;
		return true;
	}
	return false;
}

// pthread can only start C functions, not C++ functions, this func
// just calls member function

void *startRecordThread(void *pv) {
  VPBTrunk *trunk = (VPBTrunk*)pv;
  
  trunk->RecordCIDSync();
}

// DR 24/7/01 - This thread samples the CID signal between rings

void VPBTrunk::RecordCIDSync() {
	slog(SLOG_INFO) << "record CID thread" << endl;
	memset(cidbuf, 0, sizeof(cidbuf));
	slog(SLOG_INFO) << "CID buf size" << sizeof(cidbuf) << endl;
	vpb_record_buf_start(handle, VPB_LINEAR);
	vpb_record_buf_sync(handle, (char*)cidbuf, sizeof(cidbuf));
	vpb_record_buf_finish(handle);
	slog(SLOG_INFO) << "CID recording finished" << endl;

}


